Package: StatescopeR
Type: Package
Title: StatescopeR framework for discovery of cell states from cell
        type-specific gene expression profiles inferred from bulk mRNA
        profiles
Version: 0.99.36
Authors@R: c(
    person(given = "Mischa", family = "Steketee", role = c("aut", "cre"),
             email = "m.f.b.steketee@amsterdamumc.nl",
             comment = c(ORCID = "0000-0001-7138-7554")),
    person(given = "Bauke", family = "Ylstra", role = "ths",
             email = "b.ylstra@amsterdamumc.nl",
             comment = c(ORCID = "0000-0001-9479-3010")),
    person(given = "Yongsoo", family = "Kim", role = "ths",
             email = "yo.kim@amsterdamumc.nl",
             comment = c(ORCID = "0000-0002-2995-2131")),         
    person(given = 'KWF Kankerbestrijding',
                    role = 'fnd'))
Depends: R (>= 4.6.0)
Imports: S4Vectors, SummarizedExperiment, reticulate, methods,
        SingleCellExperiment, matrixStats, scran, basilisk, Matrix,
        ComplexHeatmap, ggplot2, cowplot, utils
Suggests: BiocStyle, knitr, RefManageR, rmarkdown, sessioninfo,
        scRNAseq, scuttle, testthat
Description: StatescopeR is an R wrapper around Statescope, a computational framework designed to discover cell states from cell type-specific gene expression profiles inferred from bulk RNA profiles.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
StagedInstall: no
biocViews: GeneExpression, RNASeq, SingleCell, Bayesian,
        Transcriptomics, Software
URL: https://github.com/tgac-vumc/StatescopeR
BugReports: https://github.com/tgac-vumc/StatescopeR/issues
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/StatescopeR
git_branch: devel
git_last_commit: 9120937
git_last_commit_date: 2026-04-08
Repository: Bioconductor 3.23
Date/Publication: 2026-04-08
NeedsCompilation: no
Packaged: 2026-04-09 00:47:43 UTC; biocbuild
Author: Mischa Steketee [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7138-7554>),
  Bauke Ylstra [ths] (ORCID: <https://orcid.org/0000-0001-9479-3010>),
  Yongsoo Kim [ths] (ORCID: <https://orcid.org/0000-0002-2995-2131>),
  KWF Kankerbestrijding [fnd]
Maintainer: Mischa Steketee <m.f.b.steketee@amsterdamumc.nl>

# Generated by roxygen2: do not edit by hand

export(BLADE_deconvolution)
export(Refinement)
export(StateDiscovery)
export(barplot_stateloadings)
export(create_signature)
export(fetch_signature)
export(fraction_eval)
export(fraction_heatmap)
export(gather_true_fractions)
export(select_genes)
import(ComplexHeatmap)
import(basilisk)
import(ggplot2)
import(reticulate)
import(scran)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,"logcounts<-")
importFrom(SingleCellExperiment,colData)
importFrom(SingleCellExperiment,logcounts)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(cowplot,plot_grid)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,cor)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,read.table)

<p align="center">
  <img width="424" height="200" src="https://github.com/tgac-vumc/Statescope/blob/master/img/Logo_Statescope.png">
</p>

# StatescopeR
StatescopeR is an R wrapper around Statescope, a computational framework designed to discover cell states from cell type-specific gene expression profiles inferred from bulk RNA profiles.
See https://github.com/tgac-vumc/Statescope for the original Python version.
<p align="center">
  <img width="75%" height="75%" src="https://github.com/tgac-vumc/Statescope/blob/master/img/Statescope_Overview.png">
</p>


## Installation
To install this package, start R (version "4.6") and enter: 
```
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
    
# The following initializes usage of Bioc devel
BiocManager::install(version='devel')

BiocManager::install("StatescopeR")
```

#' Python environments
statescope <- BasiliskEnvironment(
    envname = "statescope",
    pkgname = "StatescopeR",
    packages = c(
        "python==3.11",
        "anndata==0.11.4",
        "deap==1.4.3",
        "cachetools==7.0.5",
        "numba==0.62.1",
        "pandas==1.5.3",
        "joblib==1.5.3",
        "numpy==1.23.5",
        "scipy==1.15.3",
        "matplotlib==3.6.3",
        "seaborn==0.13.2",
        "scikit-learn==1.5.2",
        "torch==2.8.0", "dill==0.3.4",
        "statescope-autogenes==1.0.4.post2",
        "psutil==7.2.2"
    ),
    channels = c("anaconda", "bioconda", "conda-forge", "pytorch")
)

#' Run StateDiscovery
#'
#' \code{StateDiscovery.R} Discovers states from refined ct-specific gep
#'
#' @param Statescope SummarizedExperiment object from Statescope Refinement.
#' @param k number of cluster to choose, default is NA for automatic selection
#' @param Ncores number of cores to use for paralellization.
#' @param max_clusters maximum allowed states per cell type.
#' @param n_iter Number of initial cNMF restarts.
#' @param n_final_iter Number of final cNMF restarts.
#' @param min_cophenetic Minimum cophenetic coefficient to determine K.
#'
#' @return SummarizedExperiment object with statescores per celltype added
#' @import reticulate basilisk
#' @importFrom Matrix rowSums
#' @importFrom S4Vectors DataFrame metadata<-
#' @importFrom SummarizedExperiment assay
#' @importFrom methods is
#' @export
#'
#' @examples
#' ## Load Refined Statescope object
#' load(system.file("extdata", "example_Statescope_Refined.RData",
#'     package = "StatescopeR"
#' ))
#'
#' ## Discover states
#' Statescope <- StateDiscovery(Statescope, k = 2L, Ncores = 2L)
#'
#' ## Look at statescores and stateloadings
#' S4Vectors::metadata(Statescope)$statescores
#' S4Vectors::metadata(Statescope)$stateloadings
#'
StateDiscovery <- function(Statescope, k = NA, max_clusters = 10L, n_iter = 10L,
    n_final_iter = 100L, min_cophenetic = 0.9, Ncores = 1L) {
    if (!is(Statescope, "SummarizedExperiment")) { ## Check Statescope input
        stop("Statescope is not a SummarizedExperiment object")}
    ## start basilisk & run StateDiscovery
    proc <- basiliskStart(statescope)
    Statescope <- basiliskRun(proc, fun <- function(Statescope, max_clusters,
        n_iter, n_final_iter, min_cophenetic, Ncores) {
        ## source cNMF code
        reticulate::source_python(system.file("python/cNMF_functions.py",
            package = "StatescopeR"))
        reticulate::source_python(system.file("python/cNMF_helper_functions.py",
            package = "StatescopeR"))

        ## perform state discovery per cell type
        statescores <- list()
        stateloadings <- list()
        for (ct in names(S4Vectors::metadata(Statescope)$ct_specific_gep)) {
            ## get ct_specific_gep for state clustering
            data_scaled <- as.matrix(SummarizedExperiment::assay(
                S4Vectors::metadata(Statescope)$ct_specific_gep[[ct]],
                "weighted_gep"))
            ## Run initial NMF runs for k selection
            if (is.na(k)) {nclust <- select_k(data_scaled, max_clusters, n_iter,
                    Ncores, min_cophenetic)}
            else {nclust <- k} # or use preselected k
            ## Run final model
            final_cNMF_result <- cNMF(data_scaled, as.integer(nclust),
                n_final_iter, Ncores)
            final_H <- t(final_cNMF_result[[1]]$H)
            final_H_norm <- S4Vectors::DataFrame(final_H / rowSums(final_H))
            rownames(final_H_norm) <- colnames(Statescope)

            final_W <- S4Vectors::DataFrame(final_cNMF_result[[1]]$W)
            rownames(final_W) <- rownames(
                S4Vectors::metadata(Statescope)$ct_specific_gep[[1]])
            ## Add result to ct lists
            statescores[ct] <- final_H_norm
            stateloadings[ct] <- final_W}
        ## Add statescores to Statescope obj
        S4Vectors::metadata(Statescope)$statescores <- statescores
        S4Vectors::metadata(Statescope)$stateloadings <- stateloadings
        Statescope},Statescope = Statescope, max_clusters = max_clusters,
    n_iter = n_iter, n_final_iter = n_final_iter,
    min_cophenetic = min_cophenetic, Ncores = Ncores)
    ## stop basilisk
    basiliskStop(proc)

    return(Statescope)
}

#' Run BLADE deconvolution
#'
#' \code{BLADE_deconvolution.R} Runs BLADE to estimate cell fractions from bulk
#' mRNA
#'
#'
#' @param signature SimpleList with mu and sigma, both nGene x nCelltype
#' dataframes, respectively mean gene expression and mean-variance corrected
#' variance per cell type
#' @param bulk nSample x nGene mRNA to be deconvolved
#' @param genes subset of genes to be used for deconvolution
#' @param prior (optional) nSample x nCelltype matrix with prior fraction
#' expectations
#' @param cores number of cores to use for paralellization
#' @param Alpha BLADE Hyperparameter
#' @param Alpha0 BLADE Hyperparameter
#' @param Kappa0 BLADE Hyperparameter
#' @param sY BLADE Hyperparameter
#' @param Nrep Number of BLADE initializations
#' @param Nrepfinal Number of maximum optimization iterations
#'
#' @return SummarizedExperiment object with BLADE output and estimated fractions
#' @import reticulate basilisk
#' @importFrom SummarizedExperiment SummarizedExperiment assay
#' @importFrom S4Vectors DataFrame metadata<-
#' @importFrom methods new
#' @export
#'
#' @examples
#' if (requireNamespace("scRNAseq", quietly = TRUE)) {
#'     library(scRNAseq)
#'     library(scuttle)
#'     ## Load SegerstolpePancreas data set
#'     scRNAseq <- SegerstolpePancreasData()
#'
#'     ## remove duplicate genes
#'     scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]
#'     ## Subset to 1 healthy and 2 type 2 diabetes samples
#'     scRNAseq <- scRNAseq[, scRNAseq$individual %in% c(
#'         "H3",
#'         "T2D1", "T2D2"
#'     )]
#'     ## remove cells with no cell type label
#'     scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]
#'
#'     ## remove very rare cell types (<100 cells in total data set)
#'     celltypes_to_remove <-
#'     names(table(scRNAseq$`cell type`)[(table(scRNAseq$`cell type`) < 100)])
#'     scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]
#'
#'     ## Create pseudobulk and normalize to cp10k
#'     pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
#'     normcounts(pseudobulk) <- calculateCPM(pseudobulk) / 100
#'     pseudobulk <- as(pseudobulk, "SummarizedExperiment")
#'     rownames(pseudobulk) <- rownames(scRNAseq)
#'
#'     ## Load signature
#'     load(system.file("extdata", "example_signature.RData",
#'         package = "StatescopeR"
#'     ))
#'
#'     ## Load selected_genes
#'     load(system.file("extdata", "example_selected_genes.RData",
#'         package = "StatescopeR"
#'     ))
#'
#'     ##  Load prior
#'     load(system.file("extdata", "example_prior.RData",
#'         package = "StatescopeR"
#'     ))
#'
#'     ## Perform Deconvolution with BLADE
#'     Statescope <- BLADE_deconvolution(
#'         signature, pseudobulk, selected_genes,
#'         prior, 1L,
#'         Nrep = 1L
#'     )
#'
#'     ## show estimated fractions
#'     S4Vectors::metadata(Statescope)$fractions
#' }
BLADE_deconvolution <- function(signature, bulk, genes, prior = NULL,
                            cores = 1L, Alpha = 1L, Alpha0 = 1000L,
                            Kappa0 = 1L, sY = 1L, Nrep = 10L,Nrepfinal = 1000L){
    if (any(round(colSums(as.matrix(assay(bulk, "normcounts")))) != 10000)) {
        stop("bulk is not in counts per 10k")}
    ## make matrices from Mu, omega, bulk & prior
    Mu <- as.matrix(signature$mu[genes, ])
    Omega <- as.matrix(signature$omega[genes, ])
    bulk_matrix <- as.matrix(assay(bulk[genes, ], "normcounts"))
    if (is.null(prior)) { # keep null if no prior is given
    } else if (is.list(prior)) { # do nothing if group prior
    } else {prior <- as.matrix(prior)[colnames(bulk), colnames(signature$mu)]}
    ## start basilisk
    proc <- basiliskStart(statescope)

    ## Estimate fractions with BLADE using Basilisk
    Statescope <- basiliskRun(proc, fun = function(bulk, Mu, Omega, bulk_matrix,
    prior, Alpha, Alpha0, Kappa0, sY, Nrep, cores, Nrepfinal) {
            ## import BLADE
            reticulate::source_python(system.file("python/BLADE.py",
                package = "StatescopeR"))

            ## Run deconvolution
            result <- Framework_Iterative(Mu, Omega, bulk_matrix, Expectation =
                prior, Alpha = Alpha, Alpha0 = Alpha0, Kappa0 = Kappa0, sY = sY,
                Nrep = Nrep, Njob = cores, IterMax = Nrepfinal)
            ## make fractions DF
            fractions <- S4Vectors::DataFrame(t(result[[3]]),
                                                row.names = colnames(Mu))
            colnames(fractions) <- colnames(bulk_matrix)
            ## Make named list from result for use in refinement
            result[[1]] <- list("Alpha" = result[[1]][[1]],
                                "Beta" = result[[1]][[2]])
            ## Save BLADE parameters and fractions to metadata
            S4Vectors::metadata(bulk)$BLADE_output <- result
            S4Vectors::metadata(bulk)$fractions <- fractions
            bulk}, bulk = bulk, Mu = Mu, Omega = Omega,
    bulk_matrix = bulk_matrix,prior = prior, Alpha = Alpha, Alpha0 = Alpha0,
    Kappa0 = Kappa0, sY = sY, Nrep = Nrep, cores = cores, Nrepfinal = Nrepfinal)

    ## stop basilisk
    basiliskStop(proc)

    return(Statescope)
}

library(scRNAseq)
library(scuttle)

test_that("BLADE deconvolution works properly with prior on simulation data", {
    ## Load SegerstolpePancreas data set
    scRNAseq <- SegerstolpePancreasData()

    ## remove duplicate genes
    scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]
    ## Subset to 1 healthy and 2 type 2 diabetes samples
    scRNAseq <- scRNAseq[, scRNAseq$individual %in% c(
        "H3",
        "T2D1", "T2D2"
    )]
    ## remove cells with no cell type label
    scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]
    ## remove very rare cell types (<100 cells in total data set)
    celltypes_to_remove <- names(table(scRNAseq$`cell type`)
    [(table(scRNAseq$`cell type`) < 100)])
    scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]

    ## Create pseudobulk and normalize to cp10k
    pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
    normcounts(pseudobulk) <- calculateCPM(pseudobulk) / 100
    pseudobulk <- as(pseudobulk, "SummarizedExperiment")
    rownames(pseudobulk) <- rownames(scRNAseq)

    ##  Load signature
    load(system.file("extdata", "example_signature.RData",
        package = "StatescopeR"
    ))

    ##  Load selected genes
    load(system.file("extdata", "example_selected_genes.RData",
        package = "StatescopeR"
    ))

    ##  Load prior
    load(system.file("extdata", "example_prior.RData",
        package = "StatescopeR"
    ))

    ## Perform Deconvolution with BLADE, refine gene expression estimates
    Statescope <- BLADE_deconvolution(
        signature, pseudobulk, selected_genes,
        prior, 1L,
        Nrep = 1L ## Parallel causes workers to hang
    )

    ##  Load true fractions
    load(system.file("extdata", "example_true_fractions.RData",
        package = "StatescopeR"
    ))

    ## measure ct correlation with true fractions
    cors <- list()
    for (ct in unique(rownames(true_fractions))) {
        cor <- cor(
            as.matrix(true_fractions)[ct, ],
            as.matrix(metadata(Statescope)$fractions)
            [ct, names(true_fractions)]
        )

        ## add cor to cors
        cors[ct] <- cor
    }

    ## calculate median correlation with true fractions
    median_cor <- median(unlist(cors))

    expect_gt(median_cor, 0.4)
})

#' Fetch scRNAseq Signature
#'
#' \code{fetch_signature} Fetches signature from StatescopeData repository
#'
#'
#' @param tumor_type string choosing the tumor type from available signatures
#'  at: https://github.com/tgac-vumc/StatescopeData
#' @param n_celltypes integer choosing the number of cell types of the signature
#'  from: https://github.com/tgac-vumc/StatescopeData
#'
#' @return SimpleList with DataFrames for Mu (mean per gene per cell type) and
#' Omega (variance corrected std.dev per gene per cell type) and vector of
#' selected genes for deconvolution chosen by AutoGeneS
#' @importFrom S4Vectors DataFrame SimpleList
#' @importFrom utils read.table
#' @export
#'
#' @examples
#' signature <- fetch_signature('PDAC', 7)
#' selected_genes <- signature$selected_genes
fetch_signature <- function(tumor_type, n_celltypes) {
    ## Check input formatting
    if (!is(tumor_type, "character")) {
        stop("tumor_type is not a string")
        } else if(!is(n_celltypes, "numeric")) {
            stop("n_celltypes is not numeric")}

    # Construct file name and URL
    file_name <- paste0(tumor_type, "_Signature_", n_celltypes, "celltypes.txt")
    file_url <- paste0(
        "https://raw.githubusercontent.com/tgac-vumc/StatescopeData/master/",
        tumor_type, "/", file_name)

    # Try to read the file; handle errors
    tryCatch({con <- url(file_url, open = "r")
            on.exit(close(con), add = TRUE)  # Ensure the connection closes
            df <- read.table(
                con,
                sep = "\t",
                header = TRUE,
                row.names = "Gene",
                check.names = FALSE)},
        error = function(e) {
            stop("TumorType not specified or invalid. Check
                https://github.com/tgac-vumc/StatescopeData for valid options.",
                call. = FALSE
                )
        })

    # Adjust to Bioconductor file formats
    mu_cols <- grep('scExp', colnames(df))
    omega_cols <- grep('scVar', colnames(df))

    mu <- S4Vectors::DataFrame(df[mu_cols])
    omega <- S4Vectors::DataFrame(df[omega_cols])
    selected_genes <- rownames(df)[as.logical(df$IsMarker)]

    ## Adjust colnames of mu and omega
    colnames(mu) <- sub("^.*_", "", colnames(mu))
    colnames(omega) <- sub("^.*_", "", colnames(omega))

    return(SimpleList(mu = mu, omega = omega, selected_genes = selected_genes))
}

## usethis namespace: start
## usethis namespace: end
NULL

#' The StatescopeR package
#'
#' framework for discovery of cell states from bulk mRNA profiles
#'
#' @rdname StatescopeR-package
#' @name StatescopeR-package
#' @keywords internal
#' @aliases StatescopeR-package StatescopeR
#'
#' @section Details:
#'  StatescopeR starts from lognormalized cp10k single cell data and a bulk RNA
#'  dataset to be used for deconvolution and cell state analysis. From this
#'  point StatescopeR has some functions to create a signature and select genes
#'  for deconvolution.
#'  \code{\link{create_signature}} creates a signature of lognormalized cp10k
#'      single cell data for deconvolution and cell state analysis.
#'  \code{\link{select_genes}} selects genes which best discriminate cell types
#'      for use in deconvolution.
#'  Alternatively you can also use:
#'   \code{\link{fetch_signature}} to fetch one of the premade signatures +
#'   selected_genes from https://github.com/tgac-vumc/StatescopeData
#'   After these steps and proper cp10k normalization of the bulk mRNA
#'   deconvolution and cell state analysis can be done with the following
#'    functions:
#'  \code{\link{BLADE_deconvolution}} estimates cell fractions from bulk mRNA.
#'  \code{\link{Refinement}} refines cell type-specific gene expression profile
#'      estimates to better capture inter-sample variability.
#'  \code{\link{StateDiscovery}} discovery cell states from inferred cell
#'  type-specific gene expression profiles.
#'
#'  After these steps the following functions provide some evaluation
#'      and plotting functions:
#'  \code{\link{gather_true_fractions}} gets true cell fractions from
#'      scRNAseq data.
#'  \code{\link{fraction_eval}} plots the correlation of estimated vs true
#'      cell fractionsover all samples.
#'  \code{\link{fraction_heatmap}} plots a heatmap of estimated
#'      cell fractions.
#'  \code{\link{barplot_stateloadings}} plots a barplot showing the top n
#'      most important stateloadings.
#'

"_PACKAGE"

#' Gather true fractions
#'
#' \code{gather_true_fractions} Gathers true fractions of all cell types per
#' sample from scRNAseq data
#'
#'
#' @param scRNAseq SingleCellExperiment object of which to gather
#' fractions per sample
#' @param ids character vector with ids of samples
#' @param label_col character for the column name with the cell type labels
#'
#' @return DataFrame with fractions of all cell types per sample
#' @importFrom SingleCellExperiment colData
#' @importFrom methods is
#' @export
#'
#' @examples
#' if (requireNamespace("scRNAseq", quietly = TRUE)) {
#'     library(scRNAseq)
#'     ## Load data
#'     scRNAseq <- scRNAseq::SegerstolpePancreasData()
#'     ## Subset to 1 healthy and 3 type 2 diabetes samples
#'     scRNAseq <- scRNAseq[, scRNAseq$individual %in% c(
#'         "H3",
#'         "T2D1", "T2D2"
#'     )]
#'     ## remove NA cells
#'     scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]
#'
#'     ## remove cells with less than 100 in total cohort
#'     celltypes_to_remove <- names(table(scRNAseq$`cell type`)
#'     [(table(scRNAseq$`cell type`) < 100)])
#'     scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]
#'
#'     true_fractions <- gather_true_fractions(scRNAseq,
#'         ids = scRNAseq$individual, label_col = "cell type"
#'     )
#' }
gather_true_fractions <- function(scRNAseq, ids, label_col) {
    if (!is(scRNAseq, "SingleCellExperiment")) {
        stop("scRNAseq is not a SingleCellExperiment object")
    }
    ## Init list to save fractions
    true_fractions <- list()

    ## loop over samples
    for (sample in unique(ids)) {
        temp_scRNAseq <- scRNAseq[, ids == sample]
        temp_true_fractions <- DataFrame(table(
            colData(temp_scRNAseq)[[label_col]]
        ) / ncol(temp_scRNAseq))

        ## add True_fractions of sample to list
        true_fractions[sample] <- list(temp_true_fractions$Freq)
    }

    ## Add all samples to one DataFrame
    true_fractions <- DataFrame(true_fractions,
        row.names = temp_true_fractions$Var1
    )

    ## return DF
    return(true_fractions)
}

#' Create scRNAseq Signature
#'
#' \code{create_signature} Creates signature from scRNAseq data
#'
#'
#' @param scRNAseq SingleCellExperiment object of which to make signature
#' @param hvg_genes boolean which chooses if mu and omega should be subset to
#' highly variable genes or not
#' @param n_hvg_genes int which allows the users to choose the number of highly
#' variable genes
#' @param labels character vector for the cell type labels
#'
#' @return SimpleList DataFrames for Mu (mean per gene per cell type) and
#' Omega (variance corrected std.dev per gene per cell type)
#' @import scran
#' @importFrom matrixStats rowSds rowVars
#' @importFrom SingleCellExperiment logcounts<- logcounts
#' @importFrom methods is
#' @export
#'
#' @examples
#' if (requireNamespace("scRNAseq", quietly = TRUE)) {
#'     library(scRNAseq)
#'     library(scuttle)
#'     ## Load scRNaseq
#'     scRNAseq <- scRNAseq::SegerstolpePancreasData()
#'
#'     ## remove NA cells
#'     scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]
#'
#'     ## Normalize (cp10k) and logtransform scRNAseq
#'     cpm(scRNAseq) <- scuttle::calculateCPM(scRNAseq)
#'     SingleCellExperiment::logcounts(scRNAseq) <- log1p(cpm(scRNAseq) / 100)
#'
#'     ## Create signature
#'     signature <- create_signature(scRNAseq, labels = scRNAseq$`cell type`)
#' }
create_signature <- function(scRNAseq, hvg_genes = FALSE, n_hvg_genes = 3000L,
                                labels) {
    if (!is(scRNAseq, "SingleCellExperiment")) {
        stop("scRNAseq is not a SingleCellExperiment object")
    }
    ## init Mu, Omega & Var
    mu <- DataFrame()
    omega <- DataFrame()
    var <- DataFrame()

    for (celltype in unique(labels)) {
        ## subset scRNAseq on celltype
        temp_scRNAseq <- scRNAseq[, labels == celltype]
        ## Calculate Mu, Omega & Var
        ct_mu <- rowMeans(as.array(logcounts(temp_scRNAseq)))
        ct_omega <- rowSds(as.array(logcounts(temp_scRNAseq)))
        ct_var <- rowVars(as.array(logcounts(temp_scRNAseq)))

        ## Add ct Mu, Omega & var
        mu[celltype] <- ct_mu
        omega[celltype] <- ct_omega
        var[celltype] <- ct_var
    }

    ## Correct Omega by mean-var trend
    new_omega <- omega[, 0] ## make df with same # rows
    for (ct in colnames(omega)) {
        trend <- fitTrendVar(mu[, ct], omega[, ct])$trend
        new_omega[, ct] <- trend(mu[, ct])
    }

    ## subset on hvg_genes if true
    if (hvg_genes) {
        ## calculate per gene variance
        dec.data <- modelGeneVar(scRNAseq, assay.type = "logcounts")

        ## select hvg
        hvg_genes <- getTopHVGs(dec.data, n = n_hvg_genes)

        ## subset mu and omega
        mu <- mu[hvg_genes, ]
        new_omega <- new_omega[hvg_genes, ]
    }

    return(SimpleList(mu = mu, omega = new_omega))
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    ## see https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
    crop = NULL
)

## ----vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE----------------
## Bib setup
library("RefManageR")

## Write bibliography information
bib <- c(
    R = citation(),
    BiocStyle = citation("BiocStyle")[1],
    knitr = citation("knitr")[1],
    RefManageR = citation("RefManageR")[1],
    rmarkdown = citation("rmarkdown")[1],
    sessioninfo = citation("sessioninfo")[1],
    testthat = citation("testthat")[1],
    StatescopeR = citation("StatescopeR")[1]
)

## ----"install", eval = FALSE--------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#     install.packages("BiocManager")
# }
# 
# BiocManager::install("StatescopeR")
# 
# ## Check that you have a valid Bioconductor installation
# BiocManager::valid()

## ----'Prepare scRNAseq data'--------------------------------------------------
## Load StatescopeR & scRNAseq (for example data)
suppressMessages(library(StatescopeR))
suppressMessages(library(scRNAseq))
suppressMessages(library(scuttle))

## Load the SegerstolpePancreas data set
scRNAseq <- SegerstolpePancreasData()

## remove duplicate genes
scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]

## remove cells with no cell type label
scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

## remove very rare cell types (<100 cells in total data set)
celltypes_to_remove <-
    names(table(scRNAseq$`cell type`)[(table(scRNAseq$`cell type`) < 100)])
scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]

## ----'Prepare StatescopeR input'----------------------------------------------
## Normalize (cp10k) and logtransform scRNAseq
cpm(scRNAseq) <- calculateCPM(scRNAseq)
logcounts(scRNAseq) <- log1p(cpm(scRNAseq) / 100)

## Create scRNAseq reference/signature with 50 hvg for quick example
signature <- create_signature(scRNAseq,
    hvg_genes = TRUE, n_hvg_genes = 50L,
    labels = scRNAseq$`cell type`
)

## select subset of genes for deconvolution (30/50 hvg to make it quick)
selected_genes <- select_genes(scRNAseq, 30L, 50L,
    labels = scRNAseq$`cell type`
)

## Create pseudobulk and normalize to cp10k (logging is done within Statescope)
pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
normcounts(pseudobulk) <- calculateCPM(pseudobulk) / 100
pseudobulk <- as(pseudobulk, "SummarizedExperiment")
rownames(pseudobulk) <- rownames(scRNAseq)

## (optional) Create prior expectation for ductal cells
prior <- gather_true_fractions(scRNAseq,
    ids = scRNAseq$individual,
    label_col = "cell type"
) # Use True sc fractions for this
prior[rownames(prior) != "ductal cell", ] <- NA # Keep only ductal cells
prior <- t(prior) # Transpose it to nSample x nCelltype

## ----'Run StatescopeR'--------------------------------------------------------
## Run Deconvolution module
Statescope <- BLADE_deconvolution(signature, pseudobulk, selected_genes, prior)

## Run Refinement module
Statescope <- Refinement(Statescope, signature, pseudobulk, cores = 2L)


## Run State Discovery module (pick k=2 for quick example)
Statescope <- StateDiscovery(Statescope, k = 2L, Ncores = 2L)

## ----'Evaluate Statescope'----------------------------------------------------
## Count fractions of cells per sample in the pseudobulk
true_fractions <- gather_true_fractions(scRNAseq,
    ids = scRNAseq$individual,
    label_col = "cell type"
)

## Plot correlation and RMSE with true fractions per celltype
fraction_eval(Statescope, true_fractions)

## ----'Downstream analysis/visualizations'-------------------------------------
## Show predicted fractions
metadata(Statescope)$fractions

## Show predicted ductal cell type specific gene expression profiles
assay(metadata(Statescope)$ct_specific_gep$`ductal cell`, "weighted_gep")

## Show ductal cell state scores per sample
metadata(Statescope)$statescores$`ductal cell`

## Show ductal cell state loadings
metadata(Statescope)$stateloadings$`ductal cell`

## Plot heatmap of fractions
fraction_heatmap(Statescope)

## Plot barplot of top state loadings
barplot_stateloadings(Statescope)

## ----'Group expectations'-----------------------------------------------------
## define cell types in one group
grouped_cts <- c("alpha cell", "beta cell", "gamma cell", "delta cell")

## initialize group assignment matrix (nCelltype x nGroup) with 0's
celltype_names <- colnames(signature$mu)
group_names <- c("Group", setdiff(celltype_names, grouped_cts))
nCelltype <- length(celltype_names)
nGroup <- length(group_names)

group <- matrix(0,
    nrow = nGroup, ncol = nCelltype,
    dimnames = list(group_names, celltype_names)
)


## initialize prior fraction matrix (nSamples x nCelltypes) with NA's
nSamples <- ncol(pseudobulk)
sample_names <- colnames(pseudobulk)

prior <- matrix(
    nrow = nSamples, ncol = nGroup,
    dimnames = list(sample_names, group_names)
)


## Assign cell types to groups
for (ct in celltype_names) {
    if (ct %in% grouped_cts) {
        ## assign cell type to group
        group["Group", ct] <- 1
    } else {
        ## Assign cell type to its own group
        group[ct, ct] <- 1
    }
}

## Add grouped prior fractions to prior
true_fractions <- gather_true_fractions(scRNAseq, ids = scRNAseq$individual,
     label_col = "cell type")

prior[names(true_fractions), "Group"] <- colSums(as.matrix(
    true_fractions[grouped_cts, ]
))


## init group prior
group_prior <- list("Group" = group, "Expectation" = prior)

## Now you can just run Statescope as with a normal prior. Note we do not give
## a ductal cell prior this time
Statescope <- BLADE_deconvolution(signature, pseudobulk, selected_genes,
    group_prior,
    cores = 1L,
    Nrep = 1L
)

## ----"citation"---------------------------------------------------------------
## Citation info
citation("StatescopeR")

## ----reproduce3, echo=FALSE---------------------------------------------------
## Session info
library("sessioninfo")
options(width = 80)
session_info()

# This script was used to create the `example_scRNAseq.RData`, `example_signature.RData`,
# `example_selected_genes.RData`, `example_Statescope_Deconvolved.RData`,
# `example_Statescope_Refined.RData` and `example_Statescope_Discovered.RData` files.
# This file contains an example Statescope pipeline of which the intermediate results
# are saved for use in examples and tests.
# For this purpose the SegerstolpePancreasData from the scRNAseq package was used.
# Rare celltypes were excluded, leaving the 5 most common celltypes for analysis.
# After this exclusion, standard preprocessing was done, after which genes were
# selected with AutoGeneS and saved, before running the Statescope framework and saving all steps.
# Key package versions:
#
# scRNAseq   v2.23.0

library(StatescopeR)
library(scRNAseq)
library(scuttle)

## Load SegerstolpePancreas data set
scRNAseq <- SegerstolpePancreasData()

## remove duplicate genes
scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]

## Subset to 1 healthy and 2 type 2 diabetes samples
scRNAseq = scRNAseq[,scRNAseq$individual %in% c('H3',
                                                'T2D1','T2D2')]
## remove cells with no cell type label
scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

## remove rare cell types (<100 cells in total data set)
celltypes_to_remove <-
    names(table(scRNAseq$`cell type`)[(table(scRNAseq$`cell type`) < 100)])
scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]

## Gather true fractions and save
true_fractions <- gather_true_fractions(scRNAseq, ids = scRNAseq$individual,
                                        label_col = 'cell type')
save(true_fractions, file = 'inst/extdata/example_true_fractions.RData')

## Normalize (cp10k) and logtransform scRNAseq
cpm(scRNAseq) <- calculateCPM(scRNAseq)
logcounts(scRNAseq) <- log1p(cpm(scRNAseq)/100)

## Create pseudobulk and normalize to cp10k (logging is done within Statescope)
pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
normcounts(pseudobulk) <- calculateCPM(pseudobulk)/100
pseudobulk = as(pseudobulk, "SummarizedExperiment")
rownames(pseudobulk) = rownames(scRNAseq)

## Create scRNAseq reference/signature with 5 hvg for quick example
signature <- create_signature(scRNAseq, hvg_genes = TRUE, n_hvg_genes =  5L,
                              labels = scRNAseq$`cell type`)

save(signature, file = 'inst/extdata/example_signature.RData')

## select subset of genes for deconvolution (3/5 hvg to make it quick)
selected_genes <- select_genes(scRNAseq, 3L, 5L,
                               labels = scRNAseq$`cell type`)

save(selected_genes, file = 'inst/extdata/example_selected_genes.RData')

## (optional) Create prior expectation using True sc fractions
prior <- gather_true_fractions(scRNAseq,
            ids = scRNAseq$individual, label_col = 'cell type')
prior[rownames(prior) != "ductal cell", ] <- NA #Keep only ductal cells as prior
prior <- t(prior) # Transpose it to nSample x nCelltype

save(prior, file = 'inst/extdata/example_prior.RData')

## Perform Deconvolution with BLADE, refine gene expression estimates
Statescope <- BLADE_deconvolution(
    signature, pseudobulk, selected_genes,
    prior, 1L, Nrep = 1L
)

## Save to RData
save(Statescope, file = 'inst/extdata/example_Statescope_Deconvolved.RData')

Statescope <- Refinement(Statescope, signature, pseudobulk, 2L)

## Save to RData
save(Statescope, file = 'inst/extdata/example_Statescope_Refined.RData')

## Discover states
Statescope <- StateDiscovery(Statescope, k=2L, Ncores = 2L)

## Save to RData
save(Statescope, file = 'inst/extdata/example_Statescope_Discovered.RData')

#' Select genes using AutoGeneS
#'
#' \code{select_genes.R} select genes using AutoGeneS for deconvolution
#'
#'
#' @param scRNAseq SingleCellExperiment object to use for gene selection, should
#' be same as signature dataset
#' @param fixed_n_features integer number of genes to pick with autogenes,
#' default is NA which lets autogenes itself pick
#' @param n_hvg_genes int which allows the users to choose the number of highly
#' variable genes
#' @param labels character vector with cell type labels
#'
#' @return Vector of genes to use for deconvolution
#' @import scran basilisk reticulate
#' @importFrom Matrix rowMeans
#' @importFrom methods is
#' @importFrom SingleCellExperiment logcounts<- logcounts
#' @export
#'
#' @examples
#' if (requireNamespace("scRNAseq", quietly = TRUE)) {
#'     library(scRNAseq)
#'     library(scuttle)
#'     ## Load SegerstolpePancreas data set
#'     scRNAseq <- SegerstolpePancreasData()
#'
#'     ## remove duplicate genes
#'     scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]
#'
#'     ## Subset to 1 healthy and 2 type 2 diabetes samples
#'     scRNAseq <- scRNAseq[, scRNAseq$individual %in% c(
#'         "H3",
#'         "T2D1", "T2D2"
#'     )]
#'     ## remove cells with no cell type label
#'     scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]
#'
#'     ## remove rare cell types (<100 cells in total data set)
#'     celltypes_to_remove <-names(table(scRNAseq$`cell type`)
#'         [(table(scRNAseq$`cell type`) < 100)])
#'     scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]
#'
#'     ## remove NA cells
#'     scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]
#'
#'     ## Normalize (cp10k) and logtransform scRNAseq
#'     cpm(scRNAseq) <- scuttle::calculateCPM(scRNAseq)
#'     logcounts(scRNAseq) <- log1p(cpm(scRNAseq) / 100)
#'
#'     ## Select genes by autogenes
#'     selected_genes <- select_genes(scRNAseq, 3L,
#'         n_hvg_genes = 5L,
#'         labels = scRNAseq$`cell type`
#'     ) # 3 genes
#' }
select_genes <- function(scRNAseq, fixed_n_features = NA, n_hvg_genes = 3000L,
    labels) {
    ## Check if scRNAseq is actually a SCE
    if (!is(scRNAseq, "SingleCellExperiment")) {
        stop("scRNAseq is not a SingleCellExperiment object")
    } else if (length(labels) != ncol(scRNAseq)) {
        stop("labels are not the same length as number of cells in scRNAseq")}
    ## calculate per gene variance
    dec.data <- modelGeneVar(scRNAseq, assay.type = "logcounts")

    ## select hvg
    if (nrow(scRNAseq) < n_hvg_genes) {
        hvg_genes <- rownames(scRNAseq) ## don't select hvg_genes
    } else {
        hvg_genes <- getTopHVGs(dec.data, n = n_hvg_genes)}

    ## init centroids df
    centroids <- data.frame(row.names = hvg_genes)
    ## Calculate centroids for each celltype
    for (ct in unique(labels)) {
        ## subset scRNAseq on celltype
        temp_scRNAseq <- scRNAseq[hvg_genes, labels == ct]
        ## Calculate centroids for all genes
        centroids[ct] <- Matrix::rowMeans(logcounts(temp_scRNAseq))}

    ## start basilisk
    proc <- basiliskStart(statescope)

    ## Select genes with AutoGeneS using Basilisk
    selected_genes <- basiliskRun(proc, fun = function(centroids, ngen, seed,
    offspring_size, fixed_n_features) {
            ## import autogenes
            ag <- reticulate::import("autogenes")
            ag$init(t(centroids))
            if (is.na(fixed_n_features)) {
                ag$optimize(ngen = ngen, seed = seed,
                            offspring_size = offspring_size, verbose = FALSE)}
            else {ag$optimize(ngen = ngen, nfeatures = fixed_n_features,
                    seed = seed,mode = "fixed", offspring_size = offspring_size,
                    verbose = FALSE)}
            index <- ag$select(index = 0L)
            selected_genes <- rownames(centroids)[index]
            selected_genes}, centroids = centroids, ngen = 5000L, seed = 42L,
        offspring_size = 100L, fixed_n_features = fixed_n_features)
    ## stop basilisk
    basiliskStop(proc)

    return(selected_genes)
}

#' Run Refinement
#'
#' \code{Refinement.R} # Perform Gene Expression Refinement
#'
#' @details
#' This function takes the output from BLADE_deconvolution and refines the
#' cell type specific gene expression by reoptimizing the initial estimates.
#' It reoptimzes by fixing the estimated fractions and weighing the objective
#' function value in a way that it tries to resemble the bulk RNA expression
#' more than initially
#'
#'
#' @param Statescope SummarizedExperiment object from BLADE_deconvolution
#' @param signature SimpleList with mu and sigma, respectively mean
#' gene expression and mean-variance corrected variance per cell type
#' @param bulk mRNA to be refined
#' @param cores number of cores to use for paralellization
#'
#' @return updated SummarizedExperiment object with ct_specific_gep added
#' @import basilisk reticulate
#' @importFrom S4Vectors DataFrame metadata<- metadata SimpleList
#' @importFrom SummarizedExperiment SummarizedExperiment
#' @importFrom methods is
#' @export
#'
#' @examples
#' if (requireNamespace("scRNAseq", quietly = TRUE)) {
#'     library(scRNAseq)
#'     library(scuttle)
#'     ## Load SegerstolpePancreas data set
#'     scRNAseq <- SegerstolpePancreasData()
#'
#'     ## remove duplicate genes
#'     scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]
#'
#'     ## Subset to 1 healthy and 2 type 2 diabetes samples
#'     scRNAseq <- scRNAseq[, scRNAseq$individual %in% c(
#'         "H3",
#'         "T2D1", "T2D2"
#'     )]
#'     ## remove cells with no cell type label
#'     scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]
#'
#'     ## remove very rare cell types (<100 cells in total data set)
#'     celltypes_to_remove <- names(table(scRNAseq$`cell type`)
#'     [(table(scRNAseq$`cell type`) < 100)])
#'     scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]
#'
#'     ## Create pseudobulk and normalize to cp10k
#'     pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
#'     normcounts(pseudobulk) <- calculateCPM(pseudobulk) / 100
#'     pseudobulk <- as(pseudobulk, "SummarizedExperiment")
#'     rownames(pseudobulk) <- rownames(scRNAseq)
#'     ## Load signature
#'     load(system.file("extdata", "example_signature.RData",
#'         package = "StatescopeR"
#'     ))
#'
#'     ## Load Deconvolved Statescope object
#'     load(system.file("extdata", "example_Statescope_Deconvolved.RData",
#'         package = "StatescopeR"
#'     ))
#'
#'     ## Run Refinement
#'     Statescope <- Refinement(Statescope, signature, pseudobulk, 2L)
#'
#'     ## Show cell type specific gene expression profile estimates
#'     S4Vectors::metadata(Statescope)$ct_specific_gep
#' }
Refinement <- function(Statescope, signature, bulk, cores = 1L) {
    if (!is(Statescope, "SummarizedExperiment")) { ## Check Statescope input
        stop("Statescope is not a SummarizedExperiment object")}
    ## Prepare Refinement input
    BLADE_obj <- list("final_obj" = metadata(Statescope)$BLADE_output[[1]],
        "outs" = metadata(Statescope)$BLADE_output[[2]])
    Mu <- as.matrix(signature$mu)
    Omega <- as.matrix(signature$omega)
    genes <- rownames(Mu) # subset bulk on signature genes
    bulk_matrix <- as.matrix(assay(bulk[genes, ], "normcounts"))

    ## start basilisk & Run Refinement
    proc <- basiliskStart(statescope)
    Statescope <- basiliskRun(proc, fun = function(Statescope, BLADE_obj, Mu,
    Omega, bulk_matrix, cores) {
            ## import BLADE
            reticulate::source_python(system.file("python/BLADE.py",
                package = "StatescopeR"))
            ## Run refinement
            result <- Purify_AllGenes(BLADE_obj, Mu, Omega, bulk_matrix, cores)
            ## Add refinement output
            S4Vectors::metadata(Statescope)$Refinement_output <- result
            ## Gather ct specific gep
            ct_specific_gep <- list()
            for (i in seq_along(colnames(Mu))) {
                temp_gep <- t(result[[1]][, , i])
                ## Weight by Omega
                omega_weighted_gep <- (temp_gep - colMeans(temp_gep)) *
                    result[[2]][, i]
                ## Name samples and genes
                omega_weighted_gep_df <-
                    S4Vectors::DataFrame(omega_weighted_gep)
                colnames(omega_weighted_gep_df) <- colnames(bulk_matrix)
                rownames(omega_weighted_gep_df) <- rownames(bulk_matrix)
                ## add to ct_specific_gep list
                ct_specific_gep[colnames(Mu)[i]] <-
                    SummarizedExperiment::SummarizedExperiment(
                        assays = S4Vectors::SimpleList(weighted_gep =
                                omega_weighted_gep_df))
            }
            ## Add cell type specific gene expression to Statescope obj
            S4Vectors::metadata(Statescope)$ct_specific_gep <- ct_specific_gep
            Statescope}, Statescope = Statescope, BLADE_obj = BLADE_obj,
        Mu = Mu, Omega = Omega, bulk_matrix = bulk_matrix, cores = cores)
    ## stop basilisk
    basiliskStop(proc)

    return(Statescope)
}

#' @import ggplot2
#' @importFrom cowplot plot_grid
#' @import ComplexHeatmap
#' @importFrom stats cor setNames
#' @importFrom utils head
#'
NULL

#'
#' Create a barplot of TRUE vs estimated cfs
#'
#' Create a barplot of TRUE vs estimated cfs
#'
#' @param Statescope SummarizedExperiment object from BLADE_deconvolution
#' @param true_fractions s4 Dataframe with true fractions to compare with
#'  estimated fractions
#' @return A barplot rendered to the active graphics device
#' @importFrom S4Vectors metadata
#' @importFrom methods is
#' @export fraction_eval
#' @examples
#' ## Load True fractions
#' load(system.file("extdata", "example_true_fractions.RData",
#'     package = "StatescopeR"
#' ))
#'
#' ## Load Deconvolved Statescope object
#' load(system.file("extdata", "example_Statescope_Deconvolved.RData",
#'     package = "StatescopeR"
#' ))
#'
#' ## ## Plot fraction correlation and RMSE per ct
#' fraction_eval(Statescope, true_fractions)
fraction_eval <- function(Statescope, true_fractions) {
    if (!is(Statescope, "SummarizedExperiment")) { ## Check Statescope input
        stop("Statescope is not a SummarizedExperiment object")}
    ## measure correlation and RMSE per celltype
    eval_results <- setNames(data.frame(matrix(ncol = 3, nrow = 0)),
        c("celltype", "correlation", "RMSE"))
    for (ct in unique(rownames(true_fractions))) {
        cor_ct <- cor(as.matrix(true_fractions)[ct, ], as.matrix(
            metadata(Statescope)$fractions)[ct, names(true_fractions)])

        rmse_ct <- sqrt(mean((as.matrix(
            metadata(Statescope)$fractions)[ct, names(true_fractions)]
            - as.matrix(true_fractions)[ct, ])^2))

        ## add cor to cors
        eval_results[nrow(eval_results) + 1, ] <-
            data.frame(ct, cor_ct, rmse_ct)
    }

    ## plot correlation per celltype
    corplot <- ggplot(eval_results, aes(
        x = celltype, y = correlation,
        fill = celltype)) +
        geom_bar(stat = "identity", width = 0.95) +
        theme_bw() +
        labs(x = NULL) +
        theme(
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            legend.position = "none",
            axis.text.x = element_text(angle = 45, vjust = 0.5, hjust = 0.5))

    ## plot correlation per celltype
    rmseplot <- ggplot(eval_results, aes(
        x = celltype, y = RMSE,
        fill = celltype)) +
        geom_bar(stat = "identity", width = 0.95) +
        theme_bw() +
        labs(x = NULL) +
        theme(
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            legend.position = "none",
            axis.text.x = element_text(angle = 45, vjust = 0.5, hjust = 0.5))

    ## Plot fraction evaluation per ct
    plot_grid(corplot, rmseplot)
}

#' Create a heatmap of the estimated fractions
#'
#' Create a heatmap of the estimated fractions
#'
#' @param Statescope SummarizedExperiment object from BLADE_deconvolution
#' @param ... other parameters to pass to  [ComplexHeatmap::Heatmap()]
#' @return A heatmap rendered to the active graphics device
#' @importFrom S4Vectors metadata
#' @importFrom methods is
#' @export fraction_heatmap
#' @examples
#' ## Load Deconvolved Statescope object
#' load(system.file("extdata", "example_Statescope_Deconvolved.RData",
#'     package = "StatescopeR"
#' ))
#'
#' ## Plot fraction heatmap
#' fraction_heatmap(Statescope)
#'
fraction_heatmap <- function(Statescope, ...) {
    if (!is(Statescope, "SummarizedExperiment")) { ## Check Statescope input
        stop("Statescope is not a SummarizedExperiment object")
    }
    Heatmap(as.matrix(metadata(Statescope)$fractions),
        heatmap_legend_param = list(title = "")
    )
}


#' Create a barplot of top stateloadings
#'
#' Create a barplot of top stateloadings
#'
#' @param Statescope SummarizedExperiment object from StateDiscovery
#' @param top_n integer selecting how many genes to show per state
#' @return A barplot rendered to the active graphics device
#' @importFrom S4Vectors metadata
#' @importFrom methods is
#' @export barplot_stateloadings
#' @examples
#' #' ## Load Discovered Statescope object
#' load(system.file("extdata", "example_Statescope_Discovered.RData",
#'     package = "StatescopeR"
#' ))
#'
#' ## Plot fraction heatmap
#' barplot_stateloadings(Statescope, top_n = 1)
#'
barplot_stateloadings <- function(Statescope, top_n = 1) {
    if (!is(Statescope, "SummarizedExperiment")) { ## Check Statescope input
        stop("Statescope is not a SummarizedExperiment object")
    } else if (top_n > length(names(metadata(Statescope)$stateloadings))) {
        stop("n is bigger than number of genes")
    }
    ## init df
    plot_df <- setNames(
        data.frame(matrix(ncol = 4, nrow = 0)),
        c("celltype", "state", "gene", "score")
    )
    ## Gather top genes and scores
    for (ct in names(metadata(Statescope)$stateloadings)) {
        for (state in colnames(metadata(Statescope)$stateloadings[[ct]])) {
            indices <- head(sort(metadata(Statescope)$stateloadings[[ct]]
            [, state], index.return = TRUE, decreasing = TRUE)$ix, top_n)
            genes <- rownames(metadata(
                Statescope
            )$stateloadings[[ct]][indices, ])
            scores <- metadata(Statescope)$stateloadings[[ct]][indices, state]

            ## add to plot df
            for (i in seq(top_n)) {
                plot_df[nrow(plot_df) + 1, ] <-
                    data.frame(ct, state, genes[i], scores[i])
            }
        }
    }

    ## Make barplot
    ggplot(plot_df, aes(x = celltype, y = score, fill = state)) +
        geom_bar(position = "dodge", stat = "identity") +
        geom_text(aes(label = gene),
            position = position_dodge(width = 0.9),
            vjust = 1.3
        )
}


## Make Python and dplyr (.) functions global for check
utils::globalVariables(c(
    "Framework_Iterative", "Purify_AllGenes", "biggest_drop",
    "cNMF", "find_threshold", "select_k", ".",
    "RMSE", "celltype", "correlation", "gene", "score"
))

---
title: "Introduction to StatescopeR"
author: 
  - name: Mischa Steketee
    affiliation:
    - Department of Pathology, Cancer Center Amsterdam, Amsterdam UMC,
     Vrije Universiteit Amsterdam, Amsterdam, The Netherlands
    email: m.f.b.steketee@amsterdamumc.nl
  - name: Yongsoo Kim
    affiliation:
    - Department of Pathology, Cancer Center Amsterdam, Amsterdam UMC,
     Vrije Universiteit Amsterdam, Amsterdam, The Netherlands
    email: yo.kim@amsterdamumc.nl
output: 
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    code_folding: show
date: "`r doc_date()`"
vignette: >
  %\VignetteIndexEntry{Introduction to StatescopeR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    ## see https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
    crop = NULL
)
```

```{r vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE}
## Bib setup
library("RefManageR")

## Write bibliography information
bib <- c(
    R = citation(),
    BiocStyle = citation("BiocStyle")[1],
    knitr = citation("knitr")[1],
    RefManageR = citation("RefManageR")[1],
    rmarkdown = citation("rmarkdown")[1],
    sessioninfo = citation("sessioninfo")[1],
    testthat = citation("testthat")[1],
    StatescopeR = citation("StatescopeR")[1]
)
```

# Introduction

StatescopeR is an R wrapper around Statescope, a computational framework designed to discover cell states from cell type-specific gene expression profiles inferred from bulk RNAseq profiles. StatescopeR executes the three following steps consecutively: 1) Single cell RNAseq reference-based deconvolution 2) Refinement of inferred cell type-specific gene expression profiles 3) Cell state discovery. In addition to bulk RNAseq and a single cell RNAseq (scRNAseq) reference, StatescopeR can integrate prior expectations of (groups of) cell fractions in the bulk RNAseq. For example tumor fractions estimated by DNA copy number analysis or immune cell fractions by immunohistochemistry. Below is a manual for running StatescopeR with scRNAseq data from human pancreas cells used for both the reference and pseudobulk which is deconvolved.

# Running StatescopeR

## Installation

StatescopeR is available through Bioconductor. Install it using the following commands in R:

```{r "install", eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("StatescopeR")

## Check that you have a valid Bioconductor installation
BiocManager::valid()
```

## Prepare scRNAseq data

After installing StatescopeR, the next step is to load it along with the scuttle and the scRNAseq packages. From the latter, the SegerstolpePancreas dataset will be used as an example. This dataset will serve two purposes: 
1) Creating a scRNAseq reference. Alternatively, premade signatures can be obtained from <https://github.com/tgac-vumc/StatescopeData> using the fetch_signature function. 
2) Creating pseudobulk for deconvolution, which can also be used for evaluation, as the true cell type fractions and expression are known. Usually, bulk RNA sequencing data, with unknown cell type fractions and expression, would be used for deconvolution.

```{r 'Prepare scRNAseq data'}
## Load StatescopeR & scRNAseq (for example data)
suppressMessages(library(StatescopeR))
suppressMessages(library(scRNAseq))
suppressMessages(library(scuttle))

## Load the SegerstolpePancreas data set
scRNAseq <- SegerstolpePancreasData()

## remove duplicate genes
scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]

## remove cells with no cell type label
scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

## remove very rare cell types (<100 cells in total data set)
celltypes_to_remove <-
    names(table(scRNAseq$`cell type`)[(table(scRNAseq$`cell type`) < 100)])
scRNAseq <- scRNAseq[, !scRNAseq$`cell type` %in% celltypes_to_remove]
```

## Prepare StatescopeR input

StatescopeR takes the following inputs: 1) scRNAseq reference/signature 2) bulk RNAseq to be deconvolved 3) selected genes for deconvolution, by default this uses `r BiocStyle::Githubpkg("theislab/AutoGeneS")` (optional) Prior expectation of fractions. See [Group expectations](#group-expectations) for expectations of groups of cell types. (optional) Hyperparameters (i.e. cores for parallelization , nrepfinal for maximum number of optimization iterations etc.)

```{r 'Prepare StatescopeR input'}
## Normalize (cp10k) and logtransform scRNAseq
cpm(scRNAseq) <- calculateCPM(scRNAseq)
logcounts(scRNAseq) <- log1p(cpm(scRNAseq) / 100)

## Create scRNAseq reference/signature with 50 hvg for quick example
signature <- create_signature(scRNAseq,
    hvg_genes = TRUE, n_hvg_genes = 50L,
    labels = scRNAseq$`cell type`
)

## select subset of genes for deconvolution (30/50 hvg to make it quick)
selected_genes <- select_genes(scRNAseq, 30L, 50L,
    labels = scRNAseq$`cell type`
)

## Create pseudobulk and normalize to cp10k (logging is done within Statescope)
pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
normcounts(pseudobulk) <- calculateCPM(pseudobulk) / 100
pseudobulk <- as(pseudobulk, "SummarizedExperiment")
rownames(pseudobulk) <- rownames(scRNAseq)

## (optional) Create prior expectation for ductal cells
prior <- gather_true_fractions(scRNAseq,
    ids = scRNAseq$individual,
    label_col = "cell type"
) # Use True sc fractions for this
prior[rownames(prior) != "ductal cell", ] <- NA # Keep only ductal cells
prior <- t(prior) # Transpose it to nSample x nCelltype
```

## Run StatescopeR

Once all input data have been prepared, the (pseudo)bulk RNAseq can be deconvolved to identify transcriptional states. In short the modules work as follows: BLADE_deconvolution employs Bayesian variational inference, modelling the bulk gene expression level of each gene j in sample i by: \begin{equation}
y_{ij} = log(\sum_{t} f_{i}^t exp(x_{ij}^t)) + \epsilon_{ij}
\end{equation} with hidden variables $f_{i}^t$ and $x_{ij}^t$ denoting the fraction of cell type t for sample i and the expression of gene j for sample i in cell type j respectively. Refinement further optimizes cell type-specific gene expression profiles, $x_{ij}^t$, by fixing estimated cell fractions and putting more emphasis on capturing inter-sample heterogeneity over staying close to the single cell prior knowledge. StateDiscovery uses convex-NMF to cluster inferred cell type-specific gene expression profiles in an unsupervised manner. These modules are wrappers around the original Python functions `r BiocStyle::Githubpkg("tgac-vumc/Statescope")`.

```{r 'Run StatescopeR'}
## Run Deconvolution module
Statescope <- BLADE_deconvolution(signature, pseudobulk, selected_genes, prior)

## Run Refinement module
Statescope <- Refinement(Statescope, signature, pseudobulk, cores = 2L)


## Run State Discovery module (pick k=2 for quick example)
Statescope <- StateDiscovery(Statescope, k = 2L, Ncores = 2L)
```

## Evaluate Statescope

Since pseudobulk data derived from scRNAseq are used in this manual, it is possible to verify the accuracy of the StatescopeR output.

```{r 'Evaluate Statescope'}
## Count fractions of cells per sample in the pseudobulk
true_fractions <- gather_true_fractions(scRNAseq,
    ids = scRNAseq$individual,
    label_col = "cell type"
)

## Plot correlation and RMSE with true fractions per celltype
fraction_eval(Statescope, true_fractions)
```

## Downstream analysis/visualizations

After running StatescopeR, a wide range of analyses can be performed. In this section, we describe how to access the results and show some examples of visualizations with them.

```{r 'Downstream analysis/visualizations'}
## Show predicted fractions
metadata(Statescope)$fractions

## Show predicted ductal cell type specific gene expression profiles
assay(metadata(Statescope)$ct_specific_gep$`ductal cell`, "weighted_gep")

## Show ductal cell state scores per sample
metadata(Statescope)$statescores$`ductal cell`

## Show ductal cell state loadings
metadata(Statescope)$stateloadings$`ductal cell`

## Plot heatmap of fractions
fraction_heatmap(Statescope)

## Plot barplot of top state loadings
barplot_stateloadings(Statescope)
```

## Group expectations

When an expected fraction is available for a group of cell types rather than for individual cell types (i.e. Lymphoid cells instead of T/B cells), StatescopeR allows the use of a group prior. Here we show an example where the expected fraction of the pancreatic islet cells (alpha, beta, gamma & delta cells) together is known, but not their individual contributions.

```{r 'Group expectations'}
## define cell types in one group
grouped_cts <- c("alpha cell", "beta cell", "gamma cell", "delta cell")

## initialize group assignment matrix (nCelltype x nGroup) with 0's
celltype_names <- colnames(signature$mu)
group_names <- c("Group", setdiff(celltype_names, grouped_cts))
nCelltype <- length(celltype_names)
nGroup <- length(group_names)

group <- matrix(0,
    nrow = nGroup, ncol = nCelltype,
    dimnames = list(group_names, celltype_names)
)


## initialize prior fraction matrix (nSamples x nCelltypes) with NA's
nSamples <- ncol(pseudobulk)
sample_names <- colnames(pseudobulk)

prior <- matrix(
    nrow = nSamples, ncol = nGroup,
    dimnames = list(sample_names, group_names)
)


## Assign cell types to groups
for (ct in celltype_names) {
    if (ct %in% grouped_cts) {
        ## assign cell type to group
        group["Group", ct] <- 1
    } else {
        ## Assign cell type to its own group
        group[ct, ct] <- 1
    }
}

## Add grouped prior fractions to prior
true_fractions <- gather_true_fractions(scRNAseq, ids = scRNAseq$individual,
     label_col = "cell type")

prior[names(true_fractions), "Group"] <- colSums(as.matrix(
    true_fractions[grouped_cts, ]
))


## init group prior
group_prior <- list("Group" = group, "Expectation" = prior)

## Now you can just run Statescope as with a normal prior. Note we do not give
## a ductal cell prior this time
Statescope <- BLADE_deconvolution(signature, pseudobulk, selected_genes,
    group_prior,
    cores = 1L,
    Nrep = 1L
)
```

# Citing `StatescopeR`

If you made use of StatescopeR for your research, please use the following information to cite the package and the underlying methodology.

```{r "citation"}
## Citation info
citation("StatescopeR")
```

# Reproducibility

The `r Biocpkg("StatescopeR")` package `r Citep(bib[["StatescopeR"]])` was made possible thanks to:

-   R `r Citep(bib[["R"]])`
-   `r Biocpkg("BiocStyle")` `r Citep(bib[["BiocStyle"]])`
-   `r CRANpkg("knitr")` `r Citep(bib[["knitr"]])`
-   `r CRANpkg("RefManageR")` `r Citep(bib[["RefManageR"]])`
-   `r CRANpkg("rmarkdown")` `r Citep(bib[["rmarkdown"]])`
-   `r CRANpkg("sessioninfo")` `r Citep(bib[["sessioninfo"]])`
-   `r CRANpkg("testthat")` `r Citep(bib[["testthat"]])`

This package was developed using `r BiocStyle::Biocpkg("biocthis")`.

`R` session information.

```{r reproduce3, echo=FALSE}
## Session info
library("sessioninfo")
options(width = 80)
session_info()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StateDiscovery.R
\name{StateDiscovery}
\alias{StateDiscovery}
\title{Run StateDiscovery}
\usage{
StateDiscovery(
  Statescope,
  k = NA,
  max_clusters = 10L,
  n_iter = 10L,
  n_final_iter = 100L,
  min_cophenetic = 0.9,
  Ncores = 1L
)
}
\arguments{
\item{Statescope}{SummarizedExperiment object from Statescope Refinement.}

\item{k}{number of cluster to choose, default is NA for automatic selection}

\item{max_clusters}{maximum allowed states per cell type.}

\item{n_iter}{Number of initial cNMF restarts.}

\item{n_final_iter}{Number of final cNMF restarts.}

\item{min_cophenetic}{Minimum cophenetic coefficient to determine K.}

\item{Ncores}{number of cores to use for paralellization.}
}
\value{
SummarizedExperiment object with statescores per celltype added
}
\description{
\code{StateDiscovery.R} Discovers states from refined ct-specific gep
}
\examples{
## Load Refined Statescope object
load(system.file("extdata", "example_Statescope_Refined.RData",
    package = "StatescopeR"
))

## Discover states
Statescope <- StateDiscovery(Statescope, k = 2L, Ncores = 2L)

## Look at statescores and stateloadings
S4Vectors::metadata(Statescope)$statescores
S4Vectors::metadata(Statescope)$stateloadings

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Refinement.R
\name{Refinement}
\alias{Refinement}
\title{Run Refinement}
\usage{
Refinement(Statescope, signature, bulk, cores = 1L)
}
\arguments{
\item{Statescope}{SummarizedExperiment object from BLADE_deconvolution}

\item{signature}{SimpleList with mu and sigma, respectively mean
gene expression and mean-variance corrected variance per cell type}

\item{bulk}{mRNA to be refined}

\item{cores}{number of cores to use for paralellization}
}
\value{
updated SummarizedExperiment object with ct_specific_gep added
}
\description{
\code{Refinement.R} # Perform Gene Expression Refinement
}
\details{
This function takes the output from BLADE_deconvolution and refines the
cell type specific gene expression by reoptimizing the initial estimates.
It reoptimzes by fixing the estimated fractions and weighing the objective
function value in a way that it tries to resemble the bulk RNA expression
more than initially
}
\examples{
if (requireNamespace("scRNAseq", quietly = TRUE)) {
    library(scRNAseq)
    library(scuttle)
    ## Load SegerstolpePancreas data set
    scRNAseq <- SegerstolpePancreasData()

    ## remove duplicate genes
    scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]

    ## Subset to 1 healthy and 2 type 2 diabetes samples
    scRNAseq <- scRNAseq[, scRNAseq$individual \%in\% c(
        "H3",
        "T2D1", "T2D2"
    )]
    ## remove cells with no cell type label
    scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

    ## remove very rare cell types (<100 cells in total data set)
    celltypes_to_remove <- names(table(scRNAseq$`cell type`)
    [(table(scRNAseq$`cell type`) < 100)])
    scRNAseq <- scRNAseq[, !scRNAseq$`cell type` \%in\% celltypes_to_remove]

    ## Create pseudobulk and normalize to cp10k
    pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
    normcounts(pseudobulk) <- calculateCPM(pseudobulk) / 100
    pseudobulk <- as(pseudobulk, "SummarizedExperiment")
    rownames(pseudobulk) <- rownames(scRNAseq)
    ## Load signature
    load(system.file("extdata", "example_signature.RData",
        package = "StatescopeR"
    ))

    ## Load Deconvolved Statescope object
    load(system.file("extdata", "example_Statescope_Deconvolved.RData",
        package = "StatescopeR"
    ))

    ## Run Refinement
    Statescope <- Refinement(Statescope, signature, pseudobulk, 2L)

    ## Show cell type specific gene expression profile estimates
    S4Vectors::metadata(Statescope)$ct_specific_gep
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_genes.R
\name{select_genes}
\alias{select_genes}
\title{Select genes using AutoGeneS}
\usage{
select_genes(scRNAseq, fixed_n_features = NA, n_hvg_genes = 3000L, labels)
}
\arguments{
\item{scRNAseq}{SingleCellExperiment object to use for gene selection, should
be same as signature dataset}

\item{fixed_n_features}{integer number of genes to pick with autogenes,
default is NA which lets autogenes itself pick}

\item{n_hvg_genes}{int which allows the users to choose the number of highly
variable genes}

\item{labels}{character vector with cell type labels}
}
\value{
Vector of genes to use for deconvolution
}
\description{
\code{select_genes.R} select genes using AutoGeneS for deconvolution
}
\examples{
if (requireNamespace("scRNAseq", quietly = TRUE)) {
    library(scRNAseq)
    library(scuttle)
    ## Load SegerstolpePancreas data set
    scRNAseq <- SegerstolpePancreasData()

    ## remove duplicate genes
    scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]

    ## Subset to 1 healthy and 2 type 2 diabetes samples
    scRNAseq <- scRNAseq[, scRNAseq$individual \%in\% c(
        "H3",
        "T2D1", "T2D2"
    )]
    ## remove cells with no cell type label
    scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

    ## remove rare cell types (<100 cells in total data set)
    celltypes_to_remove <-names(table(scRNAseq$`cell type`)
        [(table(scRNAseq$`cell type`) < 100)])
    scRNAseq <- scRNAseq[, !scRNAseq$`cell type` \%in\% celltypes_to_remove]

    ## remove NA cells
    scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

    ## Normalize (cp10k) and logtransform scRNAseq
    cpm(scRNAseq) <- scuttle::calculateCPM(scRNAseq)
    logcounts(scRNAseq) <- log1p(cpm(scRNAseq) / 100)

    ## Select genes by autogenes
    selected_genes <- select_genes(scRNAseq, 3L,
        n_hvg_genes = 5L,
        labels = scRNAseq$`cell type`
    ) # 3 genes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetch_signature.R
\name{fetch_signature}
\alias{fetch_signature}
\title{Fetch scRNAseq Signature}
\usage{
fetch_signature(tumor_type, n_celltypes)
}
\arguments{
\item{tumor_type}{string choosing the tumor type from available signatures
at: https://github.com/tgac-vumc/StatescopeData}

\item{n_celltypes}{integer choosing the number of cell types of the signature
from: https://github.com/tgac-vumc/StatescopeData}
}
\value{
SimpleList with DataFrames for Mu (mean per gene per cell type) and
Omega (variance corrected std.dev per gene per cell type) and vector of
selected genes for deconvolution chosen by AutoGeneS
}
\description{
\code{fetch_signature} Fetches signature from StatescopeData repository
}
\examples{
signature <- fetch_signature('PDAC', 7)
selected_genes <- signature$selected_genes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.R
\name{fraction_eval}
\alias{fraction_eval}
\title{Create a barplot of TRUE vs estimated cfs}
\usage{
fraction_eval(Statescope, true_fractions)
}
\arguments{
\item{Statescope}{SummarizedExperiment object from BLADE_deconvolution}

\item{true_fractions}{s4 Dataframe with true fractions to compare with
estimated fractions}
}
\value{
A barplot rendered to the active graphics device
}
\description{
Create a barplot of TRUE vs estimated cfs
}
\examples{
## Load True fractions
load(system.file("extdata", "example_true_fractions.RData",
    package = "StatescopeR"
))

## Load Deconvolved Statescope object
load(system.file("extdata", "example_Statescope_Deconvolved.RData",
    package = "StatescopeR"
))

## ## Plot fraction correlation and RMSE per ct
fraction_eval(Statescope, true_fractions)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BLADE_deconvolution.R
\name{BLADE_deconvolution}
\alias{BLADE_deconvolution}
\title{Run BLADE deconvolution}
\usage{
BLADE_deconvolution(
  signature,
  bulk,
  genes,
  prior = NULL,
  cores = 1L,
  Alpha = 1L,
  Alpha0 = 1000L,
  Kappa0 = 1L,
  sY = 1L,
  Nrep = 10L,
  Nrepfinal = 1000L
)
}
\arguments{
\item{signature}{SimpleList with mu and sigma, both nGene x nCelltype
dataframes, respectively mean gene expression and mean-variance corrected
variance per cell type}

\item{bulk}{nSample x nGene mRNA to be deconvolved}

\item{genes}{subset of genes to be used for deconvolution}

\item{prior}{(optional) nSample x nCelltype matrix with prior fraction
expectations}

\item{cores}{number of cores to use for paralellization}

\item{Alpha}{BLADE Hyperparameter}

\item{Alpha0}{BLADE Hyperparameter}

\item{Kappa0}{BLADE Hyperparameter}

\item{sY}{BLADE Hyperparameter}

\item{Nrep}{Number of BLADE initializations}

\item{Nrepfinal}{Number of maximum optimization iterations}
}
\value{
SummarizedExperiment object with BLADE output and estimated fractions
}
\description{
\code{BLADE_deconvolution.R} Runs BLADE to estimate cell fractions from bulk
mRNA
}
\examples{
if (requireNamespace("scRNAseq", quietly = TRUE)) {
    library(scRNAseq)
    library(scuttle)
    ## Load SegerstolpePancreas data set
    scRNAseq <- SegerstolpePancreasData()

    ## remove duplicate genes
    scRNAseq <- scRNAseq[!duplicated(rownames(scRNAseq)), ]
    ## Subset to 1 healthy and 2 type 2 diabetes samples
    scRNAseq <- scRNAseq[, scRNAseq$individual \%in\% c(
        "H3",
        "T2D1", "T2D2"
    )]
    ## remove cells with no cell type label
    scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

    ## remove very rare cell types (<100 cells in total data set)
    celltypes_to_remove <-
    names(table(scRNAseq$`cell type`)[(table(scRNAseq$`cell type`) < 100)])
    scRNAseq <- scRNAseq[, !scRNAseq$`cell type` \%in\% celltypes_to_remove]

    ## Create pseudobulk and normalize to cp10k
    pseudobulk <- aggregateAcrossCells(scRNAseq, ids = scRNAseq$individual)
    normcounts(pseudobulk) <- calculateCPM(pseudobulk) / 100
    pseudobulk <- as(pseudobulk, "SummarizedExperiment")
    rownames(pseudobulk) <- rownames(scRNAseq)

    ## Load signature
    load(system.file("extdata", "example_signature.RData",
        package = "StatescopeR"
    ))

    ## Load selected_genes
    load(system.file("extdata", "example_selected_genes.RData",
        package = "StatescopeR"
    ))

    ##  Load prior
    load(system.file("extdata", "example_prior.RData",
        package = "StatescopeR"
    ))

    ## Perform Deconvolution with BLADE
    Statescope <- BLADE_deconvolution(
        signature, pseudobulk, selected_genes,
        prior, 1L,
        Nrep = 1L
    )

    ## show estimated fractions
    S4Vectors::metadata(Statescope)$fractions
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.R
\name{barplot_stateloadings}
\alias{barplot_stateloadings}
\title{Create a barplot of top stateloadings}
\usage{
barplot_stateloadings(Statescope, top_n = 1)
}
\arguments{
\item{Statescope}{SummarizedExperiment object from StateDiscovery}

\item{top_n}{integer selecting how many genes to show per state}
}
\value{
A barplot rendered to the active graphics device
}
\description{
Create a barplot of top stateloadings
}
\examples{
#' ## Load Discovered Statescope object
load(system.file("extdata", "example_Statescope_Discovered.RData",
    package = "StatescopeR"
))

## Plot fraction heatmap
barplot_stateloadings(Statescope, top_n = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gather_true_fractions.R
\name{gather_true_fractions}
\alias{gather_true_fractions}
\title{Gather true fractions}
\usage{
gather_true_fractions(scRNAseq, ids, label_col)
}
\arguments{
\item{scRNAseq}{SingleCellExperiment object of which to gather
fractions per sample}

\item{ids}{character vector with ids of samples}

\item{label_col}{character for the column name with the cell type labels}
}
\value{
DataFrame with fractions of all cell types per sample
}
\description{
\code{gather_true_fractions} Gathers true fractions of all cell types per
sample from scRNAseq data
}
\examples{
if (requireNamespace("scRNAseq", quietly = TRUE)) {
    library(scRNAseq)
    ## Load data
    scRNAseq <- scRNAseq::SegerstolpePancreasData()
    ## Subset to 1 healthy and 3 type 2 diabetes samples
    scRNAseq <- scRNAseq[, scRNAseq$individual \%in\% c(
        "H3",
        "T2D1", "T2D2"
    )]
    ## remove NA cells
    scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

    ## remove cells with less than 100 in total cohort
    celltypes_to_remove <- names(table(scRNAseq$`cell type`)
    [(table(scRNAseq$`cell type`) < 100)])
    scRNAseq <- scRNAseq[, !scRNAseq$`cell type` \%in\% celltypes_to_remove]

    true_fractions <- gather_true_fractions(scRNAseq,
        ids = scRNAseq$individual, label_col = "cell type"
    )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_signature.R
\name{create_signature}
\alias{create_signature}
\title{Create scRNAseq Signature}
\usage{
create_signature(scRNAseq, hvg_genes = FALSE, n_hvg_genes = 3000L, labels)
}
\arguments{
\item{scRNAseq}{SingleCellExperiment object of which to make signature}

\item{hvg_genes}{boolean which chooses if mu and omega should be subset to
highly variable genes or not}

\item{n_hvg_genes}{int which allows the users to choose the number of highly
variable genes}

\item{labels}{character vector for the cell type labels}
}
\value{
SimpleList DataFrames for Mu (mean per gene per cell type) and
Omega (variance corrected std.dev per gene per cell type)
}
\description{
\code{create_signature} Creates signature from scRNAseq data
}
\examples{
if (requireNamespace("scRNAseq", quietly = TRUE)) {
    library(scRNAseq)
    library(scuttle)
    ## Load scRNaseq
    scRNAseq <- scRNAseq::SegerstolpePancreasData()

    ## remove NA cells
    scRNAseq <- scRNAseq[, !is.na(scRNAseq$`cell type`)]

    ## Normalize (cp10k) and logtransform scRNAseq
    cpm(scRNAseq) <- scuttle::calculateCPM(scRNAseq)
    SingleCellExperiment::logcounts(scRNAseq) <- log1p(cpm(scRNAseq) / 100)

    ## Create signature
    signature <- create_signature(scRNAseq, labels = scRNAseq$`cell type`)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.R
\name{fraction_heatmap}
\alias{fraction_heatmap}
\title{Create a heatmap of the estimated fractions}
\usage{
fraction_heatmap(Statescope, ...)
}
\arguments{
\item{Statescope}{SummarizedExperiment object from BLADE_deconvolution}

\item{...}{other parameters to pass to  [ComplexHeatmap::Heatmap()]}
}
\value{
A heatmap rendered to the active graphics device
}
\description{
Create a heatmap of the estimated fractions
}
\examples{
## Load Deconvolved Statescope object
load(system.file("extdata", "example_Statescope_Deconvolved.RData",
    package = "StatescopeR"
))

## Plot fraction heatmap
fraction_heatmap(Statescope)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basilisk.R
\docType{data}
\name{statescope}
\alias{statescope}
\title{Python environments}
\format{
An object of class \code{BasiliskEnvironment} of length 1.
}
\usage{
statescope
}
\description{
Python environments
}
\keyword{datasets}
